@@ -7,4 +7,7 @@
 # Licensed under the MIT license:
 # http://www.opensource.org/licenses/mit-license
 # Copyright (c) 2011 globo.com rafael@caricio.com
-from .context import TornadoContext, TornadoHTTPContext
+from .context import (
+        TornadoContext,
+        TornadoHTTPContext,
+        IsolatedTornadoHTTPContext)
