@@ -9,8 +9,6 @@
 # Copyright (c) 2011 Rafael Caricio rafael@caricio.com
 
 import sys
-import os
-import logging
 import time
 import contextlib
 import urllib
@@ -20,10 +18,12 @@ from tornado.httpclient import AsyncHTTPClient
 from tornado.httpserver import HTTPServer
 from tornado.stack_context import NullContext
 from tornado.testing import get_unused_port
+from tornado.web import Application, URLSpec
 from pyvows import Vows
 
 from urllib3.filepost import encode_multipart_formdata
 
+
 class AsyncTestCase(object):
 
     def get_new_ioloop(self):
@@ -72,6 +72,7 @@ class AsyncTestCase(object):
         self.stop_args = None
         return result
 
+
 class AsyncHTTPTestCase(AsyncTestCase):
     def setup(self):
         self.stopped = False
@@ -79,16 +80,44 @@ class AsyncHTTPTestCase(AsyncTestCase):
         self.failure = None
         self.stop_args = None
 
+        self.io_loop = self.get_new_ioloop()
         self.http_client = AsyncHTTPClient(io_loop=self.io_loop)
-        if 'get_app' in dir(self.__class__):
-            self.io_loop = self.get_new_ioloop()
-            self.port = get_unused_port()
+
+        if hasattr(self, 'get_app') and self.get_app:
             self.app = self.get_app()
+        elif hasattr(self, 'get_handler_spec') and self.get_handler_spec:
+            spec = self.get_handler_spec()
+            if spec:
+                if isinstance(spec, tuple):
+                    if len(spec) == 3:
+                        (pattern, handler, kwargs) = spec
+                    elif len(spec) == 2:
+                        (pattern, handler) = spec
+                        kwargs = {}
+                elif isinstance(spec, URLSpec):
+                    pattern = spec.regex.pattern
+                    handler = spec.handler_class
+                    kwargs = spec.kwargs
+
+                # create an isolated version of the handler
+                self.isolated_handler = type('IsolatedHandler', (handler,), {})
+
+                self.app = Application([
+                    (pattern, self.isolated_handler, kwargs)
+                ], self.get_application_settings())
+
+        if self.app:
+            self.port = get_unused_port()
             self.http_server = HTTPServer(self.app, io_loop=self.io_loop,
-                                          **self.get_httpserver_options())
+                                            **self.get_httpserver_options())
             self.http_server.listen(self.port)
 
     def fetch(self, path, **kwargs):
+        """
+        Simple wrapper around ``http_client``. If the given ``path`` doesn't
+        start with 'http' than ``path`` is passed on to the
+        ``http_client.fetch``
+        """
         self.http_client.fetch(self.get_url(path), self.stop, **kwargs)
         return self.wait()
 
@@ -96,8 +125,9 @@ class AsyncHTTPTestCase(AsyncTestCase):
         return {}
 
     def get_url(self, path):
-        url = 'http://localhost:%s%s' % (self.port, path)
-        return url
+        if not path.startswith('http'):
+            return  'http://localhost:%s%s' % (self.port, path)
+        return path
 
     def teardown(self):
         if 'http_server' in dir(self.__class__):
@@ -105,6 +135,7 @@ class AsyncHTTPTestCase(AsyncTestCase):
         if 'http_client' in dir(self.__class__):
             self.http_client.close()
 
+
 class ParentAttributeMixin(object):
 
     def get_parent_argument(self, name):
@@ -135,6 +166,7 @@ class TornadoContext(Vows.Context, AsyncTestCase, ParentAttributeMixin):
                     'get_url',
                     'get_new_ioloop', 'stack_context', 'stop', 'wait')
 
+
 class TornadoHTTPContext(Vows.Context, AsyncHTTPTestCase, ParentAttributeMixin):
 
     def __init__(self, parent, *args, **kwargs):
@@ -144,9 +176,10 @@ class TornadoHTTPContext(Vows.Context, AsyncHTTPTestCase, ParentAttributeMixin):
 
         super(TornadoHTTPContext, self).ignore( 'get_parent_argument',
                     'get_app', 'fetch', 'get_httpserver_options',
-                    'get_url',
-                    'get_new_ioloop', 'stack_context', 'stop', 'wait',
-                    'get', 'post')
+                    'get_url', 'get_new_ioloop', 'stack_context', 'stop',
+                    'wait', 'get', 'post', 'delete', 'head', 'put',
+                    'get_handler_spec', 'get_application_settings',
+                    'get_test_handler')
 
     def setup(self):
         AsyncHTTPTestCase.setup(self)
@@ -154,17 +187,35 @@ class TornadoHTTPContext(Vows.Context, AsyncHTTPTestCase, ParentAttributeMixin):
     def teardown(self):
         AsyncHTTPTestCase.teardown(self)
 
-    def get(self, path):
-        return self.fetch(path, method="GET")
+    def get_handler_spec(self):
+        return None
 
-    def post(self, path, data={}, multipart=False):
+    def get_application_settings(self):
+        return None
+
+    def get_test_handler(self):
+        return self.isolated_handler
+
+    def get(self, path, **kwargs):
+        return self.fetch(path, method="GET", **kwargs)
+
+    def delete(self, path, **kwargs):
+        return self.fetch(path, method="DELETE")
+
+    def put(self, path, **kwargs):
+        return self.fetch(path, method="PUT", **kwargs)
+
+    def head(self, path, **kwargs):
+        return self.fetch(path, method="DELETE", **kwargs)
+
+    def post(self, path, data={}, multipart=False, **kwargs):
         """
         Convenience wrapper for the ``http_client``.
 
         :param multipart:
             If True the given ``data`` is encoded "multipart/form-data" through
             ``urllib3``
-            If the value is a tuple of two elements, then the first element is 
+            If the value is a tuple of two elements, then the first element is
             treated as the filename of the form-data section.
         """
         body = None
@@ -175,4 +226,17 @@ class TornadoHTTPContext(Vows.Context, AsyncHTTPTestCase, ParentAttributeMixin):
         else:
             body = urllib.urlencode(data, doseq=True)
 
-        return self.fetch(path, method="POST", body=body, headers=headers)
+        if 'headers' in kwargs:
+            kwargs['headers'].update(headers)
+
+        return self.fetch(path, method="POST", body=body, headers=headers,
+                **kwargs)
+
+
+class IsolatedTornadoHTTPContext(TornadoHTTPContext):
+
+    def get_handler_spec(self):
+        return self.get_parent_argument('get_handler_spec')()
+
+    def get_application_settings(self):
+        return self.get_parent_argument('get_application_settings')()
